=begin
Copyright 2012 (c) TIG.
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
###
TIG-reloadcomps.rb
###
Reloads all component definitions that have an existing external SKP 
matching their remebered 'path'.
To run type [or copy/paste] in the Ruby Console...
  TIG.reloadcomps
With the Ruby Console is open it gives a report of the progress...
Components 'made unique' but still refering the same external file would be 
merged on the reload, so they are excluded from the processing - there's a 
beep and report lists them [if any]. Re-running the tools will merge a 
duplicated definitions in steps; alternatively you might well wish to do a 
save_as on the reported components if they differ in geometry etc, so they 
then refer to their own separate external file...
It retrurns 'false' if duplicates, or 'true' if all processed.
It is NOT one-step undoable [multi-stepped].
###
Donations:
By PayPal.com to info @ revitrev.org
###
Version:
1.0 20120306 First issue.
###
=end
###
module TIG
  def self.reloadcomps()
    puts "\n  PROCESSSING"
    model=Sketchup.active_model
    defs=model.definitions
    nams=[]
    defs.to_a.each{|defn|
      next if defn.image?
      next if defn.group?
      nams<<[defn.name, defn.path]
    }
    nams.sort!
    todo=[]
    unus=[]
    nams.each{|arr|
      if todo.flatten.include?(arr[1])
        unus << arr
      else
        todo << arr
      end
    }
    todo.each{|arr|
      name=arr[0]
      path=arr[1]
      defn=defs[name]
      next unless defn and defn.valid?
      path=defn.path
      if path.empty?
        puts name+" < INTERNAL"
        next
      end
      path.tr!("\\","/")
      if not File.exist?(path)
        puts name+" < "+path+" > FILE NOT FOUND"
        next
      elsif not defn.instances[0]
        puts name+" < "+path+" > UNUSED [PURGED]"
        next
      end
      defn.name=name+(rand.to_s)
      cdef=defs.load(path)
      unless cdef and cdef.valid?
        puts name+" < "+path+" > FAILED !"
        defn.name=name
        next
      else
        puts name+" < "+path+" > RELOADED"
      end
      inst0=defn.instances[0]
      defn.instances.each{|inst|inst.definition=cdef}
      cdef.name=name
      defs.purge_unused
    }
    if unus[0]
      UI.beep
      puts "  THESE DUPLICATE-PATHS _NOT_ RELOADED..."
      unus.each{|arr|puts arr.join(" < ")}
      puts "  SUGGEST YOU SAVE THEM INTO THEIR\n  OWN SEPARATE EXTERNAL FILES FOR EDITING..."
      puts "  RE-RUNNING THIS TOOL _WILL_ MERGE DEFINITIONS!"
      return false
    end
    puts "  ALL COMPONENTS PROCESSED."
    return true
  end
end
###